@@ -1,8 +1,10 @@
+use crate::config::DateTimeConfig;
 use crate::date::MS_WATERSHED;
+use crate::util::timestamp_to_seconds_micros;
 use crate::{
     float_parse_bytes, numbers::decimal_digits, IntFloat, MicrosecondsPrecisionOverflowBehavior, TimeConfigBuilder,
 };
-use crate::{time::TimeConfig, Date, ParseError, Time};
+use crate::{Date, ParseError, Time, TimeConfig};
 use std::cmp::Ordering;
 use std::fmt;
 use std::str::FromStr;
@@ -219,6 +221,11 @@ impl DateTime {
     pub fn parse_str(str: &str) -> Result<Self, ParseError> {
         Self::parse_bytes(str.as_bytes())
     }
+
+    /// As with [`DateTime::parse_str`] but with a [`DateTimeConfig`].
+    pub fn parse_str_with_config(str: &str, config: &DateTimeConfig) -> Result<Self, ParseError> {
+        Self::parse_bytes_with_config(str.as_bytes(), config)
+    }
     /// Parse a datetime from bytes using RFC 3339 format
     ///
     /// # Arguments
@@ -320,33 +327,38 @@ impl DateTime {
     /// assert_eq!(dt.to_string(), "2022-01-01T12:13:14");
     /// ```
     pub fn parse_bytes(bytes: &[u8]) -> Result<Self, ParseError> {
-        DateTime::parse_bytes_with_config(bytes, &TimeConfigBuilder::new().build())
+        DateTime::parse_bytes_with_config(bytes, &DateTimeConfig::default())
     }
 
-    /// Same as `DateTime::parse_bytes` but supporting TimeConfig
+    /// Same as `DateTime::parse_bytes` but supporting `DateTimeConfig`
     ///
     /// # Arguments
     ///
     /// * `bytes` - The bytes to parse
-    /// * `config` - The TimeConfig to use when parsing the time portion
+    /// * `config` - The `DateTimeConfig` to use when parsing the time portion
     ///
     /// # Examples
     ///
     /// ```
-    /// use speedate::{DateTime, Date, Time, TimeConfigBuilder};
+    /// use speedate::{DateTime, Date, Time, DateTimeConfig};
     ///
-    /// let dt = DateTime::parse_bytes_with_config(b"2022-01-01T12:13:14Z", &TimeConfigBuilder::new().build()).unwrap();
+    /// let dt = DateTime::parse_bytes_with_config(
+    ///     b"2022-01-01T12:13:14Z",
+    ///     &DateTimeConfig::default(),
+    /// ).unwrap();
     /// assert_eq!(dt.to_string(), "2022-01-01T12:13:14Z");
     /// ```
-    pub fn parse_bytes_with_config(bytes: &[u8], config: &TimeConfig) -> Result<Self, ParseError> {
-        match Self::parse_bytes_rfc3339_with_config(bytes, config) {
+    pub fn parse_bytes_with_config(bytes: &[u8], config: &DateTimeConfig) -> Result<Self, ParseError> {
+        match Self::parse_bytes_rfc3339_with_config(bytes, &config.time_config) {
             Ok(d) => Ok(d),
             Err(e) => match float_parse_bytes(bytes) {
                 IntFloat::Int(int) => Self::from_timestamp_with_config(int, 0, config),
                 IntFloat::Float(float) => {
                     let timestamp_in_milliseconds = float.abs() > MS_WATERSHED as f64;
 
-                    if config.microseconds_precision_overflow_behavior == MicrosecondsPrecisionOverflowBehavior::Error {
+                    if config.time_config.microseconds_precision_overflow_behavior
+                        == MicrosecondsPrecisionOverflowBehavior::Error
+                    {
                         let decimal_digits_count = decimal_digits(bytes);
 
                         // If the number of decimal digits exceeds the maximum allowed for the timestamp precision,
@@ -368,7 +380,7 @@ impl DateTime {
                     // if seconds is negative, we round down (left on the number line), so -6.25 -> -7
                     // which allows for a positive number of microseconds to compensate back up to -6.25
                     // which is the equivalent of doing (seconds - 1) and (microseconds + 1_000_000)
-                    // like we do in Date::timestamp_watershed
+                    // like we do in timestamp_watershed
                     let seconds = timestamp_normalized.floor() as i64;
                     let microseconds = ((timestamp_normalized - seconds as f64) * 1_000_000f64).round() as u32;
 
@@ -406,20 +418,28 @@ impl DateTime {
     /// # Examples
     ///
     /// ```
-    /// use speedate::{DateTime, TimeConfigBuilder};
+    /// use speedate::{DateTime, DateTimeConfig};
     ///
-    /// let d = DateTime::from_timestamp_with_config(1_654_619_320, 123, &TimeConfigBuilder::new().build()).unwrap();
+    /// let d = DateTime::from_timestamp_with_config(
+    ///     1_654_619_320,
+    ///     123,
+    ///     &DateTimeConfig::default(),
+    /// ).unwrap();
     /// assert_eq!(d.to_string(), "2022-06-07T16:28:40.000123");
     ///
-    /// let d = DateTime::from_timestamp_with_config(1_654_619_320_123, 123_000, &TimeConfigBuilder::new().build()).unwrap();
+    /// let d = DateTime::from_timestamp_with_config(
+    ///     1_654_619_320_123,
+    ///     123_000,
+    ///     &DateTimeConfig::default(),
+    /// ).unwrap();
     /// assert_eq!(d.to_string(), "2022-06-07T16:28:40.246000");
     /// ```
     pub fn from_timestamp_with_config(
         timestamp: i64,
         timestamp_microsecond: u32,
-        config: &TimeConfig,
+        config: &DateTimeConfig,
     ) -> Result<Self, ParseError> {
-        let (mut second, extra_microsecond) = Date::timestamp_watershed(timestamp)?;
+        let (mut second, extra_microsecond) = timestamp_to_seconds_micros(timestamp, config.timestamp_unit)?;
         let mut total_microsecond = timestamp_microsecond
             .checked_add(extra_microsecond)
             .ok_or(ParseError::TimeTooLarge)?;
@@ -432,7 +452,7 @@ impl DateTime {
         let (date, time_second) = Date::from_timestamp_calc(second)?;
         Ok(Self {
             date,
-            time: Time::from_timestamp_with_config(time_second, total_microsecond, config)?,
+            time: Time::from_timestamp_with_config(time_second, total_microsecond, &config.time_config)?,
         })
     }
 
@@ -471,7 +491,7 @@ impl DateTime {
     /// assert_eq!(d.to_string(), "2022-06-07T16:28:40.246000");
     /// ```
     pub fn from_timestamp(timestamp: i64, timestamp_microsecond: u32) -> Result<Self, ParseError> {
-        Self::from_timestamp_with_config(timestamp, timestamp_microsecond, &TimeConfigBuilder::new().build())
+        Self::from_timestamp_with_config(timestamp, timestamp_microsecond, &DateTimeConfig::default())
     }
 
     /// Create a datetime from the system time. This method uses [std::time::SystemTime] to get
@@ -581,6 +601,20 @@ impl DateTime {
         self.date.timestamp() + self.time.total_seconds() as i64
     }
 
+    /// Unix timestamp in milliseconds, ignoring timezone offset.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use speedate::DateTime;
+    ///
+    /// let dt = DateTime::parse_str("1970-01-02T00:00:00.123").unwrap();
+    /// assert_eq!(dt.timestamp_ms(), 86_400_123);
+    /// ```
+    pub fn timestamp_ms(&self) -> i64 {
+        self.timestamp() * 1000 + (self.time.microsecond / 1000) as i64
+    }
+
     /// Unix timestamp assuming epoch is in zulu timezone (1970-01-01T00:00:00Z) and accounting for
     /// timezone offset.
     ///
@@ -607,4 +641,9 @@ impl DateTime {
             None => self.timestamp(),
         }
     }
+
+    /// Unix timestamp in milliseconds accounting for timezone offset.
+    pub fn timestamp_tz_ms(&self) -> i64 {
+        self.timestamp_tz() * 1000 + (self.time.microsecond / 1000) as i64
+    }
 }
